/*
 * ChessClock.c
 *
 * Created: 06.09.2017 13:36:30
 * Author : Auheniy
 */ 

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include <stdlib.h>
#include "lcd.h"
#include "bits_macros.h"

#define SH_CP 6
#define ST_CP 7
#define DS 5
#define T_START 60
#define COUNT_TIME 14
#define SOUND 2

const unsigned int decode_bin[10] = {0x0001, 0x0003, 0x0007, 0x000F, 0x001F, 0x003F, 0x007F, 0x00FF, 0x01FF, 0x03FF};
const unsigned char decode_sound[10] = {50, 70, 90, 110, 130, 150, 180, 200, 220, 240};

const unsigned char PROGMEM text0[] = "TIME: ";
		
const unsigned char def_times[14] = {1, 2, 5, 10, 15, 20, 25, 30, 35, 40, 45, 50, 55, 60};

unsigned int time_i0 = 0;
unsigned int time_i1 = 0;

unsigned char *buffer;

volatile unsigned char upd = 0, mode = 0, over = 0, mode_q = 0, position = 0, cd = 0, sd = 0, led_data[2] = {0, 0};
//mode -  ; 0 - , 1 - , 2 - .
//upd -   

unsigned char time_ms0 = 0, time_ms1 = 0;


void Write595(unsigned char a, unsigned char b, unsigned char act_q, unsigned char led_mode)
{
	unsigned char curr_data; 
	
	switch (led_mode)
	{
		case 0:
			led_data[0] = 1;
			led_data[1] = 1;
			break;
		case 1:
			led_data[0] = 0;
			led_data[1] = 1;
			break;
		case 2:
			led_data[0] = 1;
			led_data[1] = 0;
			break;
		case 3:
			led_data[0] = 0;
			led_data[1] = 0;
			break;
	}
	
	// 
	if (BitIsSet(act_q, 0))
	{
		curr_data = a;
	}
	else
	{
		curr_data = b;
	}
	
	// ()
	ClearBit(PORTD, ST_CP);
	for (unsigned char i = 0; i < 2; i++)
	{
		ClearBit(PORTD, SH_CP);
		if (led_data[i] == 1)
		{
			SetBit(PORTD, DS);
		}
		else
		{
			ClearBit(PORTD, DS);
		}
		SetBit(PORTD, SH_CP);
	}
	for (unsigned char i = 0; i < 4; i++)
	{
		ClearBit(PORTD, SH_CP);
		ClearBit(PORTD, DS);
		SetBit(PORTD, SH_CP);
	}
	
	// (, )
	for (unsigned char i = 0; i < 10; i++)
	{
		ClearBit(PORTD, SH_CP);
		if (BitIsClear(decode_bin[curr_data], i))
		{
			SetBit(PORTD, DS);
		}
		else
		{
			ClearBit(PORTD, DS);
		}
		SetBit(PORTD, SH_CP);
	}
	SetBit(PORTD, ST_CP);
	
	// 
	if (BitIsSet(act_q, 0))
	{
		ClearBit(PORTB, 5);
		SetBit(PORTB, 4);
	}
	else
	{
		ClearBit(PORTB, 4);
		SetBit(PORTB, 5);
	}
}

//, 100
ISR(TIMER1_COMPA_vect)
{
	TCNT1 = 0x00;
	
	if (cd > 0)
	{
		cd--;
	}
	
	if (sd > 0)
	{
		sd--;
	} 
	
	switch(mode)
	{
		case 1:
			if (time_i0 > 0)
			{
				time_i0--;
			}
			else
			{
				mode = 4;
				sd = 20;
				TCCR2 = 0b01101100;
				GICR &= ~((1 << INT1)|(1 << INT0));
			}
			break;
		case 2:
			if (time_i1 > 0)
			{
				time_i1--;
			}
			else
			{
				mode = 5;
				sd = 20;
				TCCR2 = 0b01101100;
				GICR &= ~((1 << INT1)|(1 << INT0));
			}
			break;
		case 4:
			InvBit(led_data[0], 0);
			if (sd == 0)
			{
				TCCR2 = 0b01101000;
			}
			break;
		case 5:
			InvBit(led_data[1], 0);
			if (sd == 0)
			{
				TCCR2 = 0b01101000;
			}
			break;
	}
}

ISR(TIMER0_OVF_vect)
{
	InvBit(mode_q, 0);
	Write595(time_ms0, time_ms1, mode_q, mode);
	
}

ISR(INT0_vect)
{
	if ((mode == 3) & (cd == 0))
	{	
		if (position < COUNT_TIME - 1)
		{
			position++;
		}
		cd = 5;
	}
	else if ((mode == 1) || (mode == 2) || (mode == 0)) 
	{
		mode = 1;
		upd = 1;
	}
}

ISR(INT1_vect)
{
	if ((mode == 3) & (cd == 0))
	{
		if (position > 0)
		{
			position--;
		}
		cd = 5;
	}
	else if ((mode == 1) || (mode == 2) || (mode == 0)) 
	{
		mode = 2;
		upd = 1;
	}
}

void DispSetTime(unsigned char pos)
{	
	BCD_2(def_times[pos]);
	LCDstring_of_flashXY(text0, 4, 1);
	LCDstring_of_sramXY(buffer, 10, 1);
}

void RegInit(void)
{
	PORTB = 0b11000000;
	DDRB = 0b00111111;
	PORTC = 0b11000000;
	DDRC = 0b00001111;
	PORTD = 0b00011111;
	DDRD = 0b11100000;
	
	TCCR2 = 0b01101000;
	OCR2 = decode_sound[SOUND];
	
	TCCR1A = 0x00;
	TCCR1B = 0x04;
	
	TCCR0 = 0x04;
	
	MCUCR |= (1 << ISC11)|(1 << ISC01);
	MCUCR &= ~((1 << ISC10)|(1 << ISC00));
	GICR |= (1 << INT1)|(1 << INT0);
	
	//OCR1A = 3125;						//  100ms ( 8;  16 - 6250)
	OCR1A = 6250;
	TIMSK = 0b00010001;
	
	TWDR = 0x00;
	
	_delay_ms(100);
	
	LCDinit();
	
	asm("sei");
}

void SetTime(unsigned char time_min)
{
	time_i0 = time_min;
	time_i1 = time_min;
	time_i0 *= 600;
	time_i1 *= 600;
}

int main(void)
{
	RegInit();
	LCDclear();
	
	buffer = BCD_GetPointerBuf();

	SetTime(5);
	
    while (1) 
    {
		if ((BitIsClear(PIND, 4)) & (cd == 0))
		{
			// ,    
			if (mode == 0)
			{
				mode = 3;
				position = 0;
			}
			else if ((mode == 1)||(mode == 2))
			{
				mode = 0;
			}	
			else if ((mode == 4) || (mode == 5))
			{
				GICR |= (1 << INT1)|(1 << INT0);
				SetTime(def_times[position]);
				mode = 0;
			}
			else
			{
				SetTime(def_times[position]);
				LCDclear();
				mode = 0;
			}
			while (BitIsClear(PIND, 4))
				;;
			cd = 5;
		}
		
		unsigned char time_s0, time_s1, time_m0, time_m1;
		
		time_ms0 = time_i0%10;
		time_s0 = (time_i0/10)%60;
		time_m0 = (time_i0/10)/60;
		
		time_ms1 = time_i1%10;
		time_s1 = (time_i1/10)%60;
		time_m1 = (time_i1/10)/60;
		
		if ((mode == 0) || (mode == 1) || (mode == 2))
		{
			BCD_2(time_m1);
			LCDstring_of_sramXY(buffer, 0, 0);
			BCD_2(time_s1);
			LCDdataXY(':', 2, 0);
			LCDstring_of_sramXY(buffer, 3, 0);
			
			BCD_2(time_m0);
			LCDstring_of_sramXY(buffer, 11, 0);
			BCD_2(time_s0);
			LCDdataXY(':', 13, 0);
			LCDstring_of_sramXY(buffer, 14, 0);
		}
		else if (mode == 3)
		{
			DispSetTime(position);
		}
    }
}

